//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  IssonStyle plugin
//
//  Copyright (C)2011 Charles Cave (charlesweb@optusnet.com.au)
//  Copyright (C)2014 Jörn Eichler (joerneichler@gmx.de)
//  Copyright (C)2012-2015 Joachim Schmitz (jojo@schmitz-digital.de)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.2
import MuseScore 1.0

MuseScore {
   version:  "1.0"
   description: "This plugin set isson-style for voice#2"
   menuPath: "Plugins.Notes.IssonStyle"


    function toggleColor(e) {//element
        e.color != "#000000" ? e.color = "#000000" : e.color = "#550000";
    }

    function toggleVisible(e) {//element
        e.visible = !(e.visible);
    }

    function toggleHeadStyle(e){//element
        // toggle style NoteHead:
        e.headType != NoteHead.HEAD_WHOLE ? e.headType = NoteHead.HEAD_WHOLE : e.headType = NoteHead.HEAD_AUTO;
        e.headGroup != NoteHead.HEAD_MI ? e.headGroup = NoteHead.HEAD_MI : e.headGroup = NoteHead.HEAD_NORMAL;
        e.small = !(e.small);   
    }

   function issonStyle(element, voice) {
      if (element.type == Element.REST){
         toggleColor(element)
         toggleVisible(element);   
      }
      else if (element.type == Element.CHORD) {
         element.stemless = true;
         console.log("element.type: " + element.type + "; stemless: " + element.stemless)   
         if (element.stem){
            toggleVisible(element.stem);
         }
         if (element.hook){
            toggleVisible(element.hook);
         }   
         if (element.beam){ 
            // beams would need special treatment as they belong to more than
            // one chord, esp. if they belong to an even number of chords,
            // so for now leave (or make) them black
            //toggleColor(element.beam);
            toggleVisible(element.beam);
         }   
         if (element.stemSlash) // Acciaccatura
            toggleVisible(element.stemSlash);
      }
      else if (element.type == Element.NOTE) {
        toggleColor(element);
        toggleHeadStyle(element);
         
        if (element.accidental){
            toggleColor(element.accidental)
         }
         for (var i = 0; i < element.dots.length; i++) {
            if (element.dots[i])
               toggleColor(element.dots[i])
         }
      }
      else
         console.log("Unknown element type: " + element.type)         
   }

   // Apply the given function to all chords/rests in selection
   // or, if nothing is selected, in the entire score
   function applyToChordsAndRestsInSelection(func) {
      var cursor = curScore.newCursor()
      cursor.rewind(1)
      var startStaff
      var endStaff
      var endTick
      var fullScore = false
      if (!cursor.segment) { // no selection
         fullScore = true
         startStaff = 0 // start with 1st staff
         endStaff = curScore.nstaves - 1 // and end with last
         }
      else {
         startStaff = cursor.staffIdx
         cursor.rewind(2)
         if (cursor.tick == 0) {
            // this happens when the selection includes
            // the last measure of the score.
            // rewind(2) goes behind the last segment (where
            // there's none) and sets tick=0
            endTick = curScore.lastSegment.tick + 1
            }
         else
            endTick = cursor.tick
         endStaff = cursor.staffIdx
         }
      console.log(startStaff + " - " + endStaff + " - " + endTick)
      for (var staff = startStaff; staff <= endStaff; staff++) {
        
          // select only voice 2 (usually isson)
          //for (var voice = 0; voice < 4; voice++) {
            cursor.rewind(1) // sets voice to 0
            var voice = 1; // !!!!!!!
            cursor.voice = voice //voice has to be set after goTo
            cursor.staffIdx = staff

            if (fullScore)
               cursor.rewind(0) // if no selection, beginning of score

            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
               if (cursor.element) {
                  if (cursor.element.type == Element.REST)
                     func(cursor.element, voice)
                  else if (cursor.element.type == Element.CHORD) {
                     //cursor.element.stemless = true;   
                     func(cursor.element, voice)
                     var graceChords = cursor.element.graceNotes;
                     for (var i = 0; i < graceChords.length; i++) {
                        // iterate through all grace chords
                        func(graceChords[i], voice)
                        var notes = graceChords[i].notes
                        for (var j = 0; j < notes.length; j++)
                           func(notes[j], voice)
                     }
                     var notes = cursor.element.notes
                     for (var i = 0; i < notes.length; i++) {
                        var note = notes[i]
                        func(note, voice)
                     }
                  } // end if CHORD
               } // end if element
               cursor.next()
            } // end while cursor
            //} // end for loop
         } // end for loop
      }

   onRun: {
      console.log("Hello, Isson")
      if (typeof curScore !== 'undefined')
         applyToChordsAndRestsInSelection(issonStyle)            
      Qt.quit()
      }
}
